/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.upgrade.internal.graph;

import com.liferay.portal.upgrade.registry.UpgradeInfo;

import java.util.List;

import org.jgrapht.EdgeFactory;

/**
 * @author Miguel Pastor
 * @author Carlos Sierra Andrés
 */
public class UpgradeProcessEdgeFactory
	implements EdgeFactory<String, UpgradeProcessEdge> {

	public UpgradeProcessEdgeFactory(List<UpgradeInfo> upgradeInfos) {
		_upgradeInfos = upgradeInfos;
	}

	@Override
	public UpgradeProcessEdge createEdge(
		String sourceVertex, String targetVertex) {

		for (UpgradeInfo upgradeInfo : _upgradeInfos) {
			String fromSchemaVersionString =
				upgradeInfo.getFromSchemaVersionString();
			String toSchemaVersionString =
				upgradeInfo.getToSchemaVersionString();

			if (fromSchemaVersionString.equals(sourceVertex) &&
				toSchemaVersionString.equals(targetVertex)) {

				return new UpgradeProcessEdge(upgradeInfo);
			}
		}

		return null;
	}

	private final List<UpgradeInfo> _upgradeInfos;

}